﻿Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput

' This example hooks up a couple of events for an AutoCAD variable.
' The variable the event tracks is named myCustomVar. There is a reg file
' that adds the variable. After entering the reg file into the registry and 
' starting AutoCAD you can enter myCustomVar at the command line. You can set the 
' value to 0 or 1. After doing a NETLOAD and loadding this example, run the
' AddVariablesEvent command. When you change the value of myCustomVar the event
' prints information to the command line. 

' There is also a command (addReg) that adds a variable named DevDaysVar. 
' After running the command you would need to restart AutoCAD to be able
' to use that variable. You could change the example so that the events 
' watch for the DevDaysVar variable. (you can also change it to watch any variable)

Public Class Class1

    Private Shared testEveHandler As TestVarEveHandler

    

    <CommandMethod("AddVariablesEvent")> _
    Public Sub AddVariablesEvent()

        addVariableEvent("DevDaysVar")
    End Sub

    Public Sub addVariableEvent(ByVal varname As String) ' add event handler for a sysvar
        Dim var As Variable = SystemObjects.Variables(varname)
        testEveHandler = New TestVarEveHandler(varname)
    End Sub

    <CommandMethod("RemoveVariablesEvent")> _
    Public Sub removeVariableEvent()  'remove event handler for a sysvar

        testEveHandler.RemoveEventHandler()

    End Sub

    <CommandMethod("addReg")> _
   Public Sub AddRegistryKey()
        'This sub adds a variable named DevDaysVarKey
        'Note: The key is not read dynamically, (only works if key exists on startup)
        Dim strRegPath As String
        strRegPath = "SOFTWARE\Autodesk\AutoCAD\R18.1\ACAD-9001:409\Variables"

        Dim acadVariablesKey As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(strRegPath, True)
        Dim DevDaysVarKey As Microsoft.Win32.RegistryKey = acadVariablesKey.CreateSubKey("DevDaysVar")

        'Setting the default value for the key
        'AutoCAD crashes on startup if (Default) does not have a value set
        DevDaysVarKey.SetValue(Nothing, "0", Microsoft.Win32.RegistryValueKind.[String])

        DevDaysVarKey.SetValue("LegacyKeyName", "General", Microsoft.Win32.RegistryValueKind.[String])

        DevDaysVarKey.SetValue("LegacyType", 4, Microsoft.Win32.RegistryValueKind.DWord)

        DevDaysVarKey.SetValue("LegacyValueName", "DevDaysVarKey", Microsoft.Win32.RegistryValueKind.[String])

        DevDaysVarKey.SetValue("LowerBound", 0, Microsoft.Win32.RegistryValueKind.DWord)

        DevDaysVarKey.SetValue("PrimaryType", 5003, Microsoft.Win32.RegistryValueKind.DWord)

        'StorageType 2 is Per Profile
        DevDaysVarKey.SetValue("StorageType", 2, Microsoft.Win32.RegistryValueKind.DWord)

        DevDaysVarKey.SetValue("UpperBound", 1, Microsoft.Win32.RegistryValueKind.DWord)

        DevDaysVarKey.Close()
        acadVariablesKey.Close()

    End Sub

End Class

Friend Class TestVarEveHandler

    Private Shared var As Variable
    Private Shared EventOn As Boolean = False

    Public Sub New(ByVal varname As String)
        If EventOn = True Then
            'which means the event handler of former variable is still on, then remove it 
            RemoveEventHandler()
        End If

        var = SystemObjects.Variables(varname)
        AddHandler var.Changed, AddressOf OnVarChanged
        AddHandler var.Changing, AddressOf OnVarChanging
        EventOn = True
    End Sub

    Private Sub OnVarChanged(ByVal sender As Object, ByVal e As VariableChangedEventArgs)

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage(String.Format(vbLf & " Variable {0} is changed.", var.Name))
        ed.WriteMessage(String.Format(vbLf & " Value = {0}", var.Value))
        ed.WriteMessage(String.Format(vbLf & " Storage = {0}", var.Storage))
        ed.WriteMessage(String.Format(vbLf & " PrimaryType = {0}", var.PrimaryType))

    End Sub

    Private Sub OnVarChanging(ByVal sender As Object, ByVal e As VariableChangingEventArgs)

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage(String.Format(vbLf & " Variable {0} is changing.", var.Name))

    End Sub

    Public Sub RemoveEventHandler()
        If EventOn = False Then
            Exit Sub
        End If
        RemoveHandler var.Changed, AddressOf OnVarChanged
        RemoveHandler var.Changing, AddressOf OnVarChanging
        EventOn = False
    End Sub

End Class


